/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_CAN4VED_H
#define IMPL_TYPE_CAN4VED_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_uint8.h"

struct CAN4VED {
    ::Float YawRateCAN{ 0.0F };
    ::Float YawAngleCAN{ 0.0F };
    ::Float SteeringWheelAngleCAN{ 0.0F };
    ::Float LatAccelerationCAN{ 0.0F };
    ::Float LongAccelerationCAN{ 0.0F };
    ::Float VelocityCurrentCAN{ 0.0F };
    ::Float VelocityINSCAN{ 0.0F };
    ::UInt8 MotorDirectionCAN{ 0U };
    ::UInt8 GearActaulCAN{ 0U };
    ::Float TemperarureIMUCAN{ 0.0F };
    ::UInt8 SteeringMotorStatusCAN{ 0U };
    ::UInt8 DriveMotorStatusCAN{ 0U };
    ::Float MotorTorqueActualCAN{ 0.0F };
    ::Float BrakePressureCAN{ 0.0F };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(YawRateCAN);
        fun(YawAngleCAN);
        fun(SteeringWheelAngleCAN);
        fun(LatAccelerationCAN);
        fun(LongAccelerationCAN);
        fun(VelocityCurrentCAN);
        fun(VelocityINSCAN);
        fun(MotorDirectionCAN);
        fun(GearActaulCAN);
        fun(TemperarureIMUCAN);
        fun(SteeringMotorStatusCAN);
        fun(DriveMotorStatusCAN);
        fun(MotorTorqueActualCAN);
        fun(BrakePressureCAN);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(YawRateCAN);
        fun(YawAngleCAN);
        fun(SteeringWheelAngleCAN);
        fun(LatAccelerationCAN);
        fun(LongAccelerationCAN);
        fun(VelocityCurrentCAN);
        fun(VelocityINSCAN);
        fun(MotorDirectionCAN);
        fun(GearActaulCAN);
        fun(TemperarureIMUCAN);
        fun(SteeringMotorStatusCAN);
        fun(DriveMotorStatusCAN);
        fun(MotorTorqueActualCAN);
        fun(BrakePressureCAN);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("YawRateCAN", YawRateCAN);
        fun("YawAngleCAN", YawAngleCAN);
        fun("SteeringWheelAngleCAN", SteeringWheelAngleCAN);
        fun("LatAccelerationCAN", LatAccelerationCAN);
        fun("LongAccelerationCAN", LongAccelerationCAN);
        fun("VelocityCurrentCAN", VelocityCurrentCAN);
        fun("VelocityINSCAN", VelocityINSCAN);
        fun("MotorDirectionCAN", MotorDirectionCAN);
        fun("GearActaulCAN", GearActaulCAN);
        fun("TemperarureIMUCAN", TemperarureIMUCAN);
        fun("SteeringMotorStatusCAN", SteeringMotorStatusCAN);
        fun("DriveMotorStatusCAN", DriveMotorStatusCAN);
        fun("MotorTorqueActualCAN", MotorTorqueActualCAN);
        fun("BrakePressureCAN", BrakePressureCAN);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("YawRateCAN", YawRateCAN);
        fun("YawAngleCAN", YawAngleCAN);
        fun("SteeringWheelAngleCAN", SteeringWheelAngleCAN);
        fun("LatAccelerationCAN", LatAccelerationCAN);
        fun("LongAccelerationCAN", LongAccelerationCAN);
        fun("VelocityCurrentCAN", VelocityCurrentCAN);
        fun("VelocityINSCAN", VelocityINSCAN);
        fun("MotorDirectionCAN", MotorDirectionCAN);
        fun("GearActaulCAN", GearActaulCAN);
        fun("TemperarureIMUCAN", TemperarureIMUCAN);
        fun("SteeringMotorStatusCAN", SteeringMotorStatusCAN);
        fun("DriveMotorStatusCAN", DriveMotorStatusCAN);
        fun("MotorTorqueActualCAN", MotorTorqueActualCAN);
        fun("BrakePressureCAN", BrakePressureCAN);
    }

    friend bool operator==(const ::CAN4VED& lhs, const ::CAN4VED& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.YawRateCAN - rhs.YawRateCAN)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.YawAngleCAN - rhs.YawAngleCAN)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.SteeringWheelAngleCAN - rhs.SteeringWheelAngleCAN)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.LatAccelerationCAN - rhs.LatAccelerationCAN)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.LongAccelerationCAN - rhs.LongAccelerationCAN)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.VelocityCurrentCAN - rhs.VelocityCurrentCAN)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.VelocityINSCAN - rhs.VelocityINSCAN)) < DBL_EPSILON) && (lhs.MotorDirectionCAN == rhs.MotorDirectionCAN) && (lhs.GearActaulCAN == rhs.GearActaulCAN) && (fabs(static_cast<double>(lhs.TemperarureIMUCAN - rhs.TemperarureIMUCAN)) < DBL_EPSILON) && (lhs.SteeringMotorStatusCAN == rhs.SteeringMotorStatusCAN) && (lhs.DriveMotorStatusCAN == rhs.DriveMotorStatusCAN) && (fabs(static_cast<double>(lhs.MotorTorqueActualCAN - rhs.MotorTorqueActualCAN)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.BrakePressureCAN - rhs.BrakePressureCAN)) < DBL_EPSILON);
    }
};


#endif // IMPL_TYPE_CAN4VED_H
